# -*- coding: utf-8 -*-
"""
Anonymize the summative assessment data of the Dutch translation
of the ASA questionnaire.

Participants were people who indicated being bilingual, having Dutch as their
first and primary language, and being fluent in English.

Since we cannot provide the non-anonymized data, you cannot run this code yourself.

Required files:
   - Prolific_Exports/Summative/prolific_export_final_eval_dutch_first_half_female_v2_2023_12_15.xlsx
   - Prolific_Exports/Summative/prolific_export_final_eval_dutch_first_half_male_other_1_2023_11_13.xlsx
   - Prolific_Exports/Summative/prolific_export_final_eval_dutch_first_half_male_other_2_v2_2023_12_07.xlsx
 
   - Prolific_Exports/Summative/prolific_export_final_eval_dutch_second_half_female_2023_11_01.xlsx
   - Prolific_Exports/Summative/prolific_export_final_eval_dutch_second_half_male_other_2023_10_26.xlsx

   - Qualtrics_Exports/Summative/Final_ASA_dutch_Summative_First_Half_final_2023_12_15.csv
   - Qualtrics_Exports/Summative/Final_ASA_Dutch_Summative_Second_Half_final_2023_11_01.csv
   
Output files:
    - Prolific_Exports/Summative/prolific_export_final_eval_dutch_first_half_final_2023_12_15_anonym.csv
    - Prolific_Exports/Summative/prolific_export_final_eval_dutch_second_half_final_2023_12_15_anonym.csv
    - Qualtrics_Exports/Summative/Final_ASA_Dutch_Summative_First_Half_final_2023_12_15_anonym.csv
    - Qualtrics_Exports/Summative/Final_ASA_German_Summative_Second_Half_final_2023_12_15_anonym.csv

Date: 2023-12-15
Author: Nele Albers
"""

import copy
import pandas as pd


def anonymize(df_q, df_p, id_extension):
    """
    Anonymize the Qualtrics and Prolific data.
    Creates a new random ID called "rand_id".

    Parameters
    ----------
    df_q : dataframe
        Qualtrics data.
    df_p : dataframe
        Prolific data.
    id_extension : string
        Extension to add to new random IDs.

    Returns
    -------
    df_q : dataframe
        Anonymized Qualtrics data.
    df_p : dataframe
        Anonymized Prolific data.

    """

    # Only use Qualtrics data of people who passed in Prolific
    df_q = df_q[df_q["PROLIFIC_PID"].isin(df_p["Participant id"].to_list())]
    df_q = df_q.reset_index(drop=True)

    # Remove columns from Qualtrics data that we cannot publish or do not need
    df_q = df_q.drop(["StartDate", "EndDate", "Status", "Progress", 
                      "Duration (in seconds)", "RecordedDate", "ResponseId", 
                      "DistributionChannel", "UserLanguage",
                      "PIV_E_G_Fluency", "SESSION_ID",
                      "STUDY_ID", "Consent2", "PROL_ID_C",
                      "TIME_VID_INTRO_First Click", "TIME_VID_INTRO_Last Click",
                      "TIME_VID_INTRO_Page Submit", "TIME_VID_INTRO_Click Count",
                      "randomflag"], 
                     axis = 1)

    # Ceate random ID
    prolific_ids_sorted = sorted(df_q["PROLIFIC_PID"].to_list())
    df_q["rand_id"] = ["P" + str(prolific_ids_sorted.index(df_q.iloc[i]["PROLIFIC_PID"]) + 1) + id_extension for i in range(len(df_q))]
    df_p["rand_id"] = ["P" + str(prolific_ids_sorted.index(df_p.iloc[i]["Participant id"]) + 1) + id_extension for i in range(len(df_p))]

    # Drop original ID columns
    df_q = df_q.drop(["PROLIFIC_PID"], axis = 1)
    df_p = df_p.drop(["Participant id"], axis = 1)

    return df_q, df_p


if __name__ == "__main__":

    # Extension to add to generated anonymized data files
    extension = "final_2023_12_15"

    #### Second half of the questionnaire

    # Load the data files we need
    df_q_2 = pd.read_csv("Qualtrics_Exports/Summative/Final_ASA_Dutch_Summative_Second_Half_final_2023_11_01.csv")
    df_p_m = pd.read_excel("Prolific_Exports/Summative/prolific_export_final_eval_dutch_second_half_male_other_2023_10_26.xlsx")
    df_p_f = pd.read_excel("Prolific_Exports/Summative/prolific_export_final_eval_dutch_second_half_female_2023_11_01.xlsx")

    # Merge Prolific data
    df_p_2 = pd.concat([df_p_m, df_p_f])

    # Anonymize
    # We need the not-anonymized Qualtrics data still for processing the first half 
    # data, so making a copy before anonymizing.
    df_q_2_anonym = copy.deepcopy(df_q_2)
    df_q_2_anonym, df_p_2 = anonymize(df_q_2_anonym, df_p_2, "_S_SH")

    # Save anonymized dataframes
    df_p_2.to_csv("Prolific_Exports/Summative/prolific_export_final_eval_dutch_second_half_" + extension + "_anonym.csv",
                  index=False)
    df_q_2_anonym.to_csv("Qualtrics_Exports/Summative/Final_ASA_Dutch_Summative_Second_Half_" + extension + "_anonym.csv")

    
    #### First half of the questionnaire

    # Load the data files we need
    df_q_1 = pd.read_csv("Qualtrics_Exports/Summative/Final_ASA_dutch_Summative_First_Half_final_2023_12_15.csv")
    df_p_m_1 = pd.read_excel("Prolific_Exports/Summative/prolific_export_final_eval_dutch_first_half_male_other_1_2023_11_13.xlsx")
    df_p_m_2 = pd.read_excel("Prolific_Exports/Summative/prolific_export_final_eval_dutch_first_half_male_other_2_v2_2023_12_07.xlsx")
    df_p_f = pd.read_excel("Prolific_Exports/Summative/prolific_export_final_eval_dutch_first_half_female_v2_2023_12_15.xlsx")

    # Merge Prolific data
    df_p_1 = pd.concat([df_p_m_1, df_p_m_2, df_p_f])

    # One person completed the second half with failed attention checks and then
    # also completed the first half. Do not want to use the first-half data from this person
    # because they had already once seen another interaction video (and maybe even the same one).
    # We replaced the Prolific ID with xxxx here since we need to anonymize everything.
    df_p_1 = df_p_1[df_p_1["Participant id"] != "xxxx"]

    # Anonymize
    df_q_1, df_p_1 = anonymize(df_q_1, df_p_1, "_S_FH")

    # Save anonymized dataframes
    df_p_1.to_csv("Prolific_Exports/Summative/prolific_export_final_eval_dutch_first_half_" + extension + "_anonym.csv",
                  index=False)
    df_q_1.to_csv("Qualtrics_Exports/Summative/Final_ASA_Dutch_Summative_First_Half_" + extension + "_anonym.csv")
